@@ -21,7 +21,7 @@ from drf_spectacular.utils import extend_schema, extend_schema_view, OpenApiPara
 from dynamicforms import fields
 from dynamicforms.action import TableAction, TablePosition
 from dynamicforms.mixins import DisplayMode
-from dynamicforms.serializers import ModelSerializer
+from dynamicforms.serializers import DynamicModelMixin, DynamicModelSerializerMixin, ModelSerializer
 from dynamicforms.template_render.layout import Column, Layout, Row
 from dynamicforms.template_render.responsive_table_layout import ResponsiveTableLayout, ResponsiveTableLayouts
 from dynamicforms.viewsets import ModelViewSet
@@ -34,13 +34,14 @@ from rest_framework.permissions import IsAuthenticated
 from rest_framework.request import Request
 from rest_framework.response import Response
 from rest_framework.serializers import Serializer
+from rest_framework.validators import UniqueValidator
 from rest_registration.exceptions import UserNotFound
 
 from django_project_base.account.constants import MERGE_USERS_QS_CK
 from django_project_base.account.middleware import ProjectNotSelectedError
 from django_project_base.account.rest.project_profiles_utils import get_project_members
 from django_project_base.base.event import UserRegisteredEvent
-from django_project_base.base.permissions import IsProjectOwner
+from django_project_base.base.permissions import is_project_owner, is_staff, is_superuser, IsProjectOwner
 from django_project_base.constants import NOTIFY_NEW_USER_SETTING_NAME
 from django_project_base.notifications.email_notification import (
     EMailNotification,
@@ -91,7 +92,10 @@ class ProfileGroupsField(fields.ManyRelatedField):
         return []
 
 
-class ProfileSerializer(ModelSerializer):
+class ProfileSerializer(DynamicModelSerializerMixin, ModelSerializer):
+    MODEL_FUNC_SETTING_NAME = "DJANGO_PROJECT_BASE_PROFILE_MODEL_AT_RUNTIME"
+    LAYOUT_FUNC_SETTING_NAME = "DJANGO_PROJECT_BASE_PROFILE_LAYOUT_AT_RUNTIME"
+
     template_context = dict(url_reverse="profile-base-project")
 
     form_titles = {
@@ -155,7 +159,7 @@ class ProfileSerializer(ModelSerializer):
         super().__init__(*args, is_filter=is_filter, **kwds)
         request = self._context["request"]
 
-        if not request.user.is_superuser:
+        if not is_superuser(request.user):
             self.fields.pop("is_staff", None)
             self.fields.pop("is_superuser", None)
 
@@ -175,7 +179,7 @@ class ProfileSerializer(ModelSerializer):
 
         if str(request.query_params.get("remove-merge-users", "false")) in tuple(
             map(str, fields.BooleanField.TRUE_VALUES)
-        ) and (request.user.is_superuser or request.user.is_staff):
+        ) and (is_superuser(request.user) or is_staff(request.user)):
             self.actions.actions.append(
                 TableAction(
                     TablePosition.ROW_END,
@@ -186,7 +190,11 @@ class ProfileSerializer(ModelSerializer):
                 ),
             )
 
-        if request.user.is_superuser or request.user.is_staff:
+        if (
+            is_superuser(request.user)
+            or is_staff(request.user)
+            or is_project_owner(request.user, request.selected_project)
+        ):
             self.actions.actions.append(
                 TableAction(
                     TablePosition.HEADER,
@@ -201,6 +209,13 @@ class ProfileSerializer(ModelSerializer):
         # we have made it so that str(UserProfile) returns full_name, but possibly decorated with status (Klubis)
         return str(obj)
 
+    def to_internal_value(self, data):
+        if not data.get("username") and data.get("email"):
+            data["username"] = data.get("email")
+        if not self.instance and not data.get("password"):
+            data["password_invalid"] = True
+        return super().to_internal_value(data)
+
     def get_is_impersonated(self, obj):
         try:
             request = self.context["request"]
@@ -253,6 +268,20 @@ class ProfileRegisterSerializer(ProfileSerializer):
         )
         exclude = ProfileSerializer.Meta.exclude + ("avatar",)
 
+    def to_internal_value(self, data):
+        # Pogledam če obstaja user z istim mailom.
+        # Če obstaja, pogledam, če je to new_user.
+        # Če je, potem za username UniqueValidator v queryset dam exclude za ta id.
+        if not self.instance:
+            if email := data.get("email"):
+                user = swapper.load_model("django_project_base", "Profile").objects.filter(email=email).first()
+                if user and user.is_new_user:
+                    for validator in self.fields["username"].validators:
+                        if isinstance(validator, UniqueValidator):
+                            validator.queryset = validator.queryset.exclude(id=user.id)
+
+        return super().to_internal_value(data)
+
     def validate(self, attrs):
         super().validate(attrs)
         errors = {}
@@ -292,11 +321,12 @@ class ProfileViewPermissions(BasePermissions):
     create=extend_schema(exclude=True),
     update=extend_schema(exclude=True),
 )
-class ProfileViewSet(ModelViewSet):
+class ProfileViewSet(DynamicModelMixin, ModelViewSet):
     filter_backends = [filters.SearchFilter]
     search_fields = search_fields
     permission_classes = (ProfileViewPermissions,)
     pagination_class = ModelViewSet.generate_paged_loader(30, ["un_sort", "id"])
+    MODEL_FUNC_SETTING_NAME = "DJANGO_PROJECT_BASE_PROFILE_MODEL_AT_RUNTIME"
 
     def get_permissions(self):
         if self.action == "register_account":
@@ -313,7 +343,11 @@ class ProfileViewSet(ModelViewSet):
             return super().get_permissions()
 
     def get_queryset(self):
-        return get_project_members(self.request)
+        profile_model = DynamicModelMixin.determine_model_at_runtime_static(
+            self.request, func_name=getattr(settings, ProfileViewSet.MODEL_FUNC_SETTING_NAME, None)
+        )
+
+        return get_project_members(self.request, profile_model=profile_model)
 
     def get_serializer_class(self):
         if self.request.query_params.get("select", "") == "1":
@@ -409,8 +443,18 @@ class ProfileViewSet(ModelViewSet):
         )
         serializer.is_valid(raise_exception=True)
 
-        if get_user_model().objects.filter(email=serializer.validated_data["email"]).exists():
-            raise ValidationError({"email": _("User with this email already exists.")})
+        user = (
+            get_user_model()
+            .objects.filter(email=serializer.validated_data["email"])
+            .select_related("userprofile")
+            .first()
+        )
+        if user:
+            if not user.userprofile or not user.userprofile.is_new_user:
+                raise ValidationError({"email": _("User with this email already exists.")})
+            serializer.instance = user
+            user.userprofile.password_invalid = False
+            user.userprofile.save(update_fields=["password_invalid"])
 
         user = serializer.save()
         user.set_password(request.data["password"])
@@ -561,7 +605,7 @@ class ProfileViewSet(ModelViewSet):
         },
     )
     def destroy(self, request, *args, **kwargs):
-        if self.request.user.is_superuser or self.request.user.is_staff:
+        if is_superuser(request.user) or is_staff(request.user):
             return super().destroy(request, *args, **kwargs)
         raise exceptions.PermissionDenied
 
