@@ -11,7 +11,6 @@ from django.db import transaction
 from django.db.models import Model
 from django.http import Http404
 from django.shortcuts import get_object_or_404
-from django.utils.module_loading import import_string
 from django.utils.translation import gettext as _
 from drf_spectacular.utils import extend_schema, OpenApiResponse
 from dynamicforms import fields
@@ -34,7 +33,7 @@ from django_project_base.base.event import (
     SmsSenderChangedEvent,
 )
 from django_project_base.base.models import BaseProjectSettings
-from django_project_base.base.permissions import CreateAny, IsProjectOwnerOrReadOnly
+from django_project_base.base.permissions import CreateAny, is_staff, is_superuser, IsProjectOwnerOrReadOnly
 from django_project_base.constants import EMAIL_SENDER_ID_SETTING_NAME, SMS_SENDER_ID_SETTING_NAME
 from django_project_base.utils import get_pk_name
 
@@ -64,12 +63,9 @@ class ProjectViewSet(DynamicModelMixin, ModelViewSet):
 
     @staticmethod
     def _get_queryset_for_request(request):
-        try:
-            # had to copy this from the mixin because we don't have self here
-            model_func = import_string(getattr(settings, ProjectViewSet.MODEL_FUNC_SETTING_NAME, None))
-            model = model_func(None, request)
-        except ImportError:
-            model = None
+        model = DynamicModelMixin.determine_model_at_runtime_static(
+            request, func_name=getattr(settings, ProjectViewSet.MODEL_FUNC_SETTING_NAME, None)
+        )
 
         if not model:
             model = swapper.load_model("django_project_base", "Project")
@@ -236,7 +232,7 @@ class ProjectSettingsSerializer(ModelSerializer):
             ),
         )
         request = self.context.get("request")
-        if request and (request.user.is_superuser or request.user.is_staff):
+        if request and (is_superuser(request.user) or is_staff(request.user)):
             # TODO: https://taiga.velis.si/project/velis74-dynamic-forms/issue/837
             self.actions.actions.append(
                 TableAction(
@@ -328,6 +324,11 @@ class ProjectSettingsSerializer(ModelSerializer):
 
 class ProjectSettingsViewSet(ModelViewSet):
     permission_classes = [IsAuthenticated]
+    # Pagination potrebujem, zato, da se sploh pokažejo zapisi v tabeli...
+    #  Drugače funkcija replace_rows nič ne naradi
+    #    - DF pričakuje da pride pagination like object (mora vsebovati "records", kjer so notri zapisi).
+    #  Potrebujem pa vse postavke billinga, ker jih ob shranjevanju pošljem skupaj z glavo.
+    pagination_class = ModelViewSet.generate_paged_loader(page_size=1000, ordering=["id"])
 
     def get_serializer_class(self):
         if (
@@ -374,7 +375,9 @@ class ProjectSettingsViewSet(ModelViewSet):
         req = super().initialize_request(request, *args, **kwargs)
         if req.method.upper() not in SAFE_METHODS:
             try:
-                req.data["project"] = request.selected_project.pk
+                # Swagger mock request pri inicializaciji swaggerja ne gre skozi middleware...
+                # in zato request sploh nima selected_project attributa
+                req.data["project"] = request.selected_project.pk if hasattr(request, "selected_project") else 0
             except ProjectNotSelectedError as e:
                 raise NotFound(e.message)
         return req
@@ -390,6 +393,10 @@ class ProjectSettingsViewSet(ModelViewSet):
         except ProjectNotSelectedError:
             return self.get_serializer().Meta.model.objects.none()
 
+    def filter_queryset_field(self, queryset, field, value):
+        filter_field = "value" if field == "table_value" else field
+        return super().filter_queryset_field(queryset, filter_field, value)
+
     def handle_create_validation_exception(self, e, request, *args, **kwargs):
         if getattr(e, "model-validation", False):
             raise ValidationError({e.detail: e.default_code})
@@ -460,7 +467,7 @@ class ProjectSettingsViewSet(ModelViewSet):
         url_path="confirm-setting-active",
     )
     def confirm_setting_active(self, request) -> Response:
-        if not (request.user.is_superuser or request.user.is_staff):
+        if not (is_superuser(request.user) or is_staff(request.user)):
             raise PermissionDenied
         setting = get_object_or_404(self.get_serializer_class().Meta.model, pk=self._get_pk_from_request(request))
         call_command(
